@@ -9,6 +9,7 @@
 # This script will convert a chess position in the FEN
 # format to an ascii representation of the position.
 
+from __future__ import print_function
 import sys,string,os
 
 os.close(0)
@@ -26,24 +27,23 @@ comp=string.split(line,'/')
 cont=1
 margin= " "*6
 
-print margin+'   +'+"-"*15+'+'
+print (margin+'   +'+"-"*15+'+')
 for i in range(8):
-
     cont = cont + 1
     tmp=""
     for j in comp[i]:
-	if j>='0' and j <= '9':
-	    for k in range(int(j)):
-		cont = cont + 1
-		x, mod = divmod(cont,2)
-		if mod : tmp = tmp + '| '
-		else : tmp = tmp + '|*'
-	else :
-	    tmp = tmp + '|' + j
-	    cont = cont + 1
+        if j>='0' and j <= '9':
+            for k in range(int(j)):
+                cont = cont + 1
+                x, mod = divmod(cont,2)
+                if mod : tmp = tmp + '| '
+                else : tmp = tmp + '|*'
+        else :
+            tmp = tmp + '|' + j
+            cont = cont + 1
 
     row = 8 - i
-    print margin, row, tmp+"|"
+    print (margin, row, tmp+"|")
 
-print margin+'   +'+"-"*15+'+'
-print margin+'    a b c d e f g h '
+print (margin+'   +'+"-"*15+'+')
+print (margin+'    a b c d e f g h ')
