@@ -509,6 +509,9 @@ enum FuncCode
 	LFUN_TAB_GROUP_PREVIOUS,        // daniel 20220130
 	LFUN_BIBTEX_DATABASE_LIST,      // bpiwowar, 20221218
 	LFUN_STATISTICS_REFERENCE_CLAMP,// sanda, 20240324
+	LFUN_REFERENCE_INSERT,          // spitz, 20240728
+	// 400
+	LFUN_REFERENCE_TO_PARAGRAPH,    // spitz, 20240728
 	LFUN_LASTACTION                 // end of the table
 };
 
