@@ -15,67 +15,123 @@
 
 #include "Language.h"
 
+#include "BufferParams.h"
 #include "Encoding.h"
+#include "LaTeXFonts.h"
 #include "Lexer.h"
 #include "LyXRC.h"
 
 #include "support/debug.h"
+#include "support/docstring.h"
 #include "support/FileName.h"
 #include "support/filetools.h"
 #include "support/lassert.h"
 #include "support/lstrings.h"
+#include "support/qstring_helpers.h"
 #include "support/Messages.h"
 
+#include <QLocale>
+#include <QString>
+
 using namespace std;
 using namespace lyx::support;
 
 namespace lyx {
 
 Languages languages;
-Language const * ignore_language = 0;
-Language const * default_language = 0;
-Language const * latex_language = 0;
-Language const * reset_language = 0;
+Language const * ignore_language = nullptr;
+Language const * default_language = nullptr;
+Language const * latex_language = nullptr;
+Language const * reset_language = nullptr;
 
 
 bool Language::isPolyglossiaExclusive() const
 {
-	return babel().empty() && !polyglossia().empty() && requires().empty();
+	return babel().empty() && !polyglossia().empty() && required().empty();
 }
 
 
 bool Language::isBabelExclusive() const
 {
-	return !babel().empty() && polyglossia().empty() && requires().empty();
+	return !babel().empty() && polyglossia().empty() && required().empty();
 }
 
 
-docstring const Language::translateLayout(string const & m) const
+docstring const Language::translateLayout(string const & msg) const
 {
-	if (m.empty())
+	if (msg.empty())
 		return docstring();
 
-	if (!isAscii(m)) {
-		lyxerr << "Warning: not translating `" << m
+	if (!isAscii(msg)) {
+		lyxerr << "Warning: not translating `" << msg
 		       << "' because it is not pure ASCII.\n";
-		return from_utf8(m);
+		return from_utf8(msg);
 	}
 
-	TranslationMap::const_iterator it = layoutTranslations_.find(m);
+	TranslationMap::const_iterator it = layoutTranslations_.find(msg);
 	if (it != layoutTranslations_.end())
 		return it->second;
 
-	docstring t = from_ascii(m);
+	docstring t = from_ascii(msg);
 	cleanTranslation(t);
 	return t;
 }
 
 
+string Language::fontenc(BufferParams const & params) const
+{
+	// Don't use LaTeX fonts, so just return the language's preferred
+	// (although this is not used with nonTeXFonts anyway).
+	if (params.useNonTeXFonts)
+		return fontenc_.front() == "ASCII" ? "T1" : fontenc_.front();
+
+	// Determine optimal font encoding
+	// We check whether the used rm font supports an encoding our language supports
+	LaTeXFont const & lf =
+		theLaTeXFonts().getLaTeXFont(from_ascii(params.fontsRoman()));
+	vector<string> const & lfe = lf.fontencs();
+	for (auto & fe : fontenc_) {
+		// ASCII means: support all T* encodings plus OT1
+		if (fe == "ASCII") {
+			for (auto & afe : lfe) {
+				if (afe == "OT1" || prefixIs(afe, "T"))
+					// we found a suitable one; return that.
+					return afe;
+			}
+		}
+		// For other encodings, just check whether the font supports it
+		if (lf.hasFontenc(fe))
+			return fe;
+	}
+	// We did not find a suitable one; just take the first in the list,
+	// the priorized one (which is "T1" for ASCII).
+	return fontenc_.front() == "ASCII" ? "T1" : fontenc_.front();
+}
+
+
+string Language::dateFormat(size_t i) const
+{
+	if (i > dateformats_.size())
+		return string();
+	return dateformats_.at(i);
+}
+
+
+docstring Language::decimalSeparator() const
+{
+	if (lyxrc.default_decimal_sep == "locale") {
+		QLocale loc = QLocale(toqstr(code()));
+		return qstring_to_ucs4(QString(loc.decimalPoint()));
+	}
+	return from_utf8(lyxrc.default_decimal_sep);
+}
+
+
 bool Language::readLanguage(Lexer & lex)
 {
 	enum LanguageTags {
-		LA_AS_BABELOPTS = 1,
-		LA_BABELNAME,
+		LA_BABELNAME = 1,
+		LA_DATEFORMATS,
 		LA_ENCODING,
 		LA_END,
 		LA_FONTENC,
@@ -86,17 +142,22 @@ bool Language::readLanguage(Lexer & lex)
 		LA_LANG_VARIETY,
 		LA_POLYGLOSSIANAME,
 		LA_POLYGLOSSIAOPTS,
+		LA_XINDYNAME,
 		LA_POSTBABELPREAMBLE,
-		LA_QUOTESTYLE,
 		LA_PREBABELPREAMBLE,
+		LA_PROVIDES,
 		LA_REQUIRES,
-		LA_RTL
+		LA_QUOTESTYLE,
+		LA_RTL,
+		LA_WORDWRAP,
+		LA_ACTIVECHARS
 	};
 
 	// Keep these sorted alphabetically!
 	LexerKeyword languageTags[] = {
-		{ "asbabeloptions",       LA_AS_BABELOPTS },
+		{ "activechars",          LA_ACTIVECHARS },
 		{ "babelname",            LA_BABELNAME },
+		{ "dateformats",          LA_DATEFORMATS },
 		{ "encoding",             LA_ENCODING },
 		{ "end",                  LA_END },
 		{ "fontencoding",         LA_FONTENC },
@@ -109,9 +170,12 @@ bool Language::readLanguage(Lexer & lex)
 		{ "polyglossiaopts",      LA_POLYGLOSSIAOPTS },
 		{ "postbabelpreamble",    LA_POSTBABELPREAMBLE },
 		{ "prebabelpreamble",     LA_PREBABELPREAMBLE },
+		{ "provides",             LA_PROVIDES },
 		{ "quotestyle",           LA_QUOTESTYLE },
 		{ "requires",             LA_REQUIRES },
-		{ "rtl",                  LA_RTL }
+		{ "rtl",                  LA_RTL },
+		{ "wordwrap",             LA_WORDWRAP },
+		{ "xindyname",            LA_XINDYNAME }
 	};
 
 	bool error = false;
@@ -137,9 +201,6 @@ bool Language::readLanguage(Lexer & lex)
 		case LA_END: // end of structure
 			finished = true;
 			break;
-		case LA_AS_BABELOPTS:
-			lex >> as_babel_options_;
-			break;
 		case LA_BABELNAME:
 			lex >> babel_;
 			break;
@@ -149,15 +210,32 @@ bool Language::readLanguage(Lexer & lex)
 		case LA_POLYGLOSSIAOPTS:
 			lex >> polyglossia_opts_;
 			break;
+		case LA_XINDYNAME:
+			lex >> xindy_;
+			break;
 		case LA_QUOTESTYLE:
 			lex >> quote_style_;
 			break;
+		case LA_ACTIVECHARS:
+			lex >> active_chars_;
+			break;
 		case LA_ENCODING:
 			lex >> encodingStr_;
 			break;
-		case LA_FONTENC:
-			lex >> fontenc_;
+		case LA_FONTENC: {
+			lex.eatLine();
+			vector<string> const fe =
+				getVectorFromString(lex.getString(true), "|");
+			fontenc_.insert(fontenc_.end(), fe.begin(), fe.end());
 			break;
+		}
+		case LA_DATEFORMATS: {
+			lex.eatLine();
+			vector<string> const df =
+				getVectorFromString(trim(lex.getString(true), "\""), "|");
+			dateformats_.insert(dateformats_.end(), df.begin(), df.end());
+			break;
+		}
 		case LA_GUINAME:
 			lex >> display_;
 			break;
@@ -182,11 +260,17 @@ bool Language::readLanguage(Lexer & lex)
 				lex.getLongString(from_ascii("EndPreBabelPreamble"));
 			break;
 		case LA_REQUIRES:
-			lex >> requires_;
+			lex >> required_;
+			break;
+		case LA_PROVIDES:
+			lex >> provides_;
 			break;
 		case LA_RTL:
 			lex >> rightToLeft_;
 			break;
+		case LA_WORDWRAP:
+			lex >> word_wrap_;
+			break;
 		}
 	}
 	lex.popTable();
@@ -196,10 +280,9 @@ bool Language::readLanguage(Lexer & lex)
 
 bool Language::read(Lexer & lex)
 {
-	as_babel_options_ = 0;
-	encoding_ = 0;
-	internal_enc_ = 0;
-	rightToLeft_ = 0;
+	encoding_ = nullptr;
+	internal_enc_ = false;
+	rightToLeft_ = false;
 
 	if (!lex.next()) {
 		lex.printError("No name given for language: `$$Token'.");
@@ -218,17 +301,23 @@ bool Language::read(Lexer & lex)
 		encoding_ = encodings.fromLyXName("iso8859-1");
 		LYXERR0("Unknown encoding " << encodingStr_);
 	}
+	if (fontenc_.empty())
+		fontenc_.push_back("ASCII");
+	if (dateformats_.empty()) {
+		dateformats_.push_back("MMMM dd, yyyy");
+		dateformats_.push_back("MMM dd, yyyy");
+		dateformats_.push_back("M/d/yyyy");
+	}
 	return true;
 }
 
 
 void Language::readLayoutTranslations(Language::TranslationMap const & trans, bool replace)
 {
-	TranslationMap::const_iterator const end = trans.end();
-	for (TranslationMap::const_iterator it = trans.begin(); it != end; ++it) {
+	for (auto const & t : trans) {
 		if (replace
-			|| layoutTranslations_.find(it->first) == layoutTranslations_.end())
-			layoutTranslations_[it->first] = it->second;
+		    || layoutTranslations_.find(t.first) == layoutTranslations_.end())
+			layoutTranslations_[t.first] = t.second;
 	}
 }
 
@@ -257,22 +346,22 @@ void Languages::read(FileName const & filename)
 			break;
 		if (l.lang() == "latex") {
 			// Check if latex language was not already defined.
-			LASSERT(latex_language == 0, continue);
+			LASSERT(latex_language == nullptr, continue);
 			static const Language latex_lang = l;
 			latex_language = &latex_lang;
 		} else if (l.lang() == "ignore") {
 			// Check if ignore language was not already defined.
-			LASSERT(ignore_language == 0, continue);
+			LASSERT(ignore_language == nullptr, continue);
 			static const Language ignore_lang = l;
 			ignore_language = &ignore_lang;
 		} else
-			languagelist[l.lang()] = l;
+			languagelist_[l.lang()] = l;
 	}
 
 	default_language = getLanguage("english");
 	if (!default_language) {
 		LYXERR0("Default language \"english\" not found!");
-		default_language = &(*languagelist.begin()).second;
+		default_language = &(*languagelist_.begin()).second;
 		LYXERR0("Using \"" << default_language->lang() << "\" instead!");
 	}
 
@@ -304,6 +393,7 @@ bool readTranslations(Lexer & lex, Language::TranslationMap & trans)
 enum Match {
 	NoMatch,
 	ApproximateMatch,
+	VeryApproximateMatch,
 	ExactMatch
 };
 
@@ -326,12 +416,56 @@ Match match(string const & code, Language const & lang)
 	if ((code.size() == 2) && (langcode.size() > 2)
 		&& (code + '_' == langcode.substr(0, 3)))
 		return ApproximateMatch;
+	if (code.substr(0,2) == langcode.substr(0,2))
+		return VeryApproximateMatch;
 	return NoMatch;
 }
 
 } // namespace
 
 
+
+Language const * Languages::getFromCode(string const & code) const
+{
+	// 1/ exact match with any known language
+	for (auto const & l : languagelist_) {
+		if (match(code, l.second) == ExactMatch)
+			return &l.second;
+	}
+
+	// 2/ approximate with any known language
+	for (auto const & l : languagelist_) {
+		if (match(code, l.second) == ApproximateMatch)
+			return &l.second;
+	}
+	return nullptr;
+}
+
+
+Language const * Languages::getFromCode(string const & code,
+			set<Language const *> const & tryfirst) const
+{
+	// 1/ exact match with tryfirst list
+	for (auto const * lptr : tryfirst) {
+		if (match(code, *lptr) == ExactMatch)
+			return lptr;
+	}
+
+	// 2/ approximate match with tryfirst list
+	for (auto const * lptr : tryfirst) {
+		Match const m = match(code, *lptr);
+		if (m == ApproximateMatch || m == VeryApproximateMatch)
+			return lptr;
+	}
+
+	// 3/ stricter match in all languages
+	return getFromCode(code);
+
+	LYXERR0("Unknown language `" << code << "'");
+	return nullptr;
+}
+
+
 void Languages::readLayoutTranslations(support::FileName const & filename)
 {
 	Lexer lex;
@@ -339,10 +473,7 @@ void Languages::readLayoutTranslations(support::FileName const & filename)
 	lex.setContext("Languages::read");
 
 	// 1) read all translations (exact and approximate matches) into trans
-	typedef std::map<string, Language::TranslationMap> TransMap;
-	TransMap trans;
-	LanguageList::iterator const lbeg = languagelist.begin();
-	LanguageList::iterator const lend = languagelist.end();
+	std::map<string, Language::TranslationMap> trans;
 	while (lex.isOK()) {
 		if (!lex.checkFor("Translation")) {
 			if (lex.isOK())
@@ -352,13 +483,7 @@ void Languages::readLayoutTranslations(support::FileName const & filename)
 		if (!lex.next(true))
 			break;
 		string const code = lex.getString();
-		bool found = false;
-		for (LanguageList::iterator lit = lbeg; lit != lend; ++lit) {
-			if (match(code, lit->second) != NoMatch) {
-				found = true;
-				break;
-			}
-		}
+		bool found = getFromCode(code);
 		if (!found) {
 			lex.printError("Unknown language `" + code + "'");
 			break;
@@ -372,15 +497,12 @@ void Languages::readLayoutTranslations(support::FileName const & filename)
 
 	// 2) merge all translations into the languages
 	// exact translations overwrite approximate ones
-	TransMap::const_iterator const tbeg = trans.begin();
-	TransMap::const_iterator const tend = trans.end();
-	for (TransMap::const_iterator tit = tbeg; tit != tend; ++tit) {
-		for (LanguageList::iterator lit = lbeg; lit != lend; ++lit) {
-			Match const m = match(tit->first, lit->second);
+	for (auto & tr : trans) {
+		for (auto & lang : languagelist_) {
+			Match const m = match(tr.first, lang.second);
 			if (m == NoMatch)
 				continue;
-			lit->second.readLayoutTranslations(tit->second,
-			                                   m == ExactMatch);
+			lang.second.readLayoutTranslations(tr.second, m == ExactMatch);
 		}
 	}
 
@@ -393,8 +515,8 @@ Language const * Languages::getLanguage(string const & language) const
 		return reset_language;
 	if (language == "ignore")
 		return ignore_language;
-	const_iterator it = languagelist.find(language);
-	return it == languagelist.end() ? reset_language : &it->second;
+	const_iterator it = languagelist_.find(language);
+	return it == languagelist_.end() ? reset_language : &it->second;
 }
 
 
