@@ -12,13 +12,9 @@
 
 #include "factory.h"
 
-#include "Buffer.h"
-#include "BufferParams.h"
-#include "FloatList.h"
 #include "FuncRequest.h"
 #include "Lexer.h"
 #include "LyX.h"
-#include "TextClass.h"
 
 #include "insets/InsetBibitem.h"
 #include "insets/InsetBibtex.h"
@@ -26,6 +22,7 @@
 #include "insets/InsetBranch.h"
 #include "insets/InsetCaption.h"
 #include "insets/InsetCitation.h"
+#include "insets/InsetCounter.h"
 #include "insets/InsetFlex.h"
 #include "insets/InsetERT.h"
 #include "insets/InsetListings.h"
@@ -37,6 +34,7 @@
 #include "insets/InsetHyperlink.h"
 #include "insets/InsetInclude.h"
 #include "insets/InsetIndex.h"
+#include "insets/InsetIndexMacro.cpp"
 #include "insets/InsetInfo.h"
 #include "insets/InsetIPA.h"
 #include "insets/InsetIPAMacro.h"
@@ -50,6 +48,7 @@
 #include "insets/InsetArgument.h"
 #include "insets/InsetPhantom.h"
 #include "insets/InsetPreview.h"
+#include "insets/InsetQuotes.h"
 #include "insets/InsetRef.h"
 #include "insets/InsetScript.h"
 #include "insets/InsetSeparator.h"
@@ -59,19 +58,19 @@
 #include "insets/InsetVSpace.h"
 #include "insets/InsetWrap.h"
 
-#include "mathed/MathMacroTemplate.h"
+#include "mathed/InsetMathMacroTemplate.h"
 #include "mathed/InsetMathHull.h"
 
 #include "frontends/alert.h"
 
 #include "support/debug.h"
-#include "support/lstrings.h"
 #include "support/ExceptionMessage.h"
-
-#include "support/lassert.h"
+#include "support/lstrings.h"
+#include "support/unique_ptr.h"
 
 #include <sstream>
 
+
 using namespace std;
 using namespace lyx::support;
 
@@ -97,6 +96,8 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 				inp.kind = InsetNewpageParams::CLEARPAGE;
 			else if (name == "cleardoublepage")
 				inp.kind = InsetNewpageParams::CLEARDOUBLEPAGE;
+			else if (name == "nopagebreak")
+				inp.kind = InsetNewpageParams::NOPAGEBREAK;
 			return new InsetNewpage(inp);
 		}
 
@@ -154,11 +155,21 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 			string const arg2 = cmd.getArg(1);
 			if (arg1 != "deco") {
 				LYXERR0("LFUN_IPAMACRO_INSERT: wrong argument");
-				return 0;
+				return nullptr;
 			}
 			return new InsetIPADeco(buf, arg2);
 		}
 
+		case LFUN_INDEXMACRO_INSERT: {
+			string const arg = cmd.getArg(0);
+			if (arg != "see" && arg != "seealso"
+			    && arg != "subentry" && arg != "sortkey") {
+				LYXERR0("LFUN_INDEXMACRO_INSERT: wrong argument");
+				return nullptr;
+			}
+			return new InsetIndexMacro(buf, arg);
+		}
+
 		case LFUN_ERT_INSERT:
 			return new InsetERT(buf);
 
@@ -175,7 +186,7 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 			string arg = cmd.getArg(0);
 			if (arg.empty()) {
 				LYXERR0("argument-insert needs an argument!");
-				return 0;
+				return nullptr;
 			}
 			return new InsetArgument(buf, arg);
 		}
@@ -211,11 +222,11 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 			if (argument == "figure" || argument == "table")
 				return new InsetWrap(buf, argument);
 			lyxerr << "Non-existent wrapfig type: " << argument << endl;
-			return 0;
+			return nullptr;
 		}
 
 		case LFUN_INDEX_INSERT: {
-			docstring arg = cmd.argument();
+			docstring const & arg = cmd.argument();
 			return new InsetIndex(buf, InsetIndexParams(arg));
 		}
 
@@ -230,7 +241,7 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 
 		case LFUN_TABULAR_INSERT: {
 			if (cmd.argument().empty())
-				return 0;
+				return nullptr;
 			istringstream ss(to_utf8(cmd.argument()));
 			int r = 0, c = 0;
 			ss >> r >> c;
@@ -262,7 +273,6 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 
 		case LFUN_INFO_INSERT: {
 			InsetInfo * inset = new InsetInfo(buf, to_utf8(cmd.argument()));
-			inset->updateInfo();
 			return inset;
 		}
 
@@ -281,95 +291,101 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 			switch (code) {
 			case NO_CODE:
 				lyxerr << "No such inset '" << name << "'.";
-				return 0;
-			
+				return nullptr;
+
 			case BIBITEM_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetBibitem(buf, icp);
 			}
-			
+
 			case BIBTEX_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetBibtex(buf, icp);
 			}
-			
+
 			case CITE_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetCitation(buf, icp);
 			}
-			
+
+			case COUNTER_CODE: {
+				InsetCommandParams icp(code);
+				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
+				return new InsetCounter(buf, icp);
+			}
+
 			case ERT_CODE: {
 				return new InsetERT(buf,
 					InsetERT::string2params(to_utf8(cmd.argument())));
 			}
-			
+
 			case EXTERNAL_CODE: {
 				InsetExternalParams iep;
 				InsetExternal::string2params(to_utf8(cmd.argument()), *buf, iep);
-				auto_ptr<InsetExternal> inset(new InsetExternal(buf));
+				auto inset = make_unique<InsetExternal>(buf);
 				inset->setBuffer(*buf);
 				inset->setParams(iep);
 				return inset.release();
 			}
-			
+
 			case GRAPHICS_CODE: {
 				InsetGraphicsParams igp;
 				InsetGraphics::string2params(to_utf8(cmd.argument()), *buf, igp);
-				auto_ptr<InsetGraphics> inset(new InsetGraphics(buf));
+				auto inset = make_unique<InsetGraphics>(buf);
 				inset->setParams(igp);
 				return inset.release();
 			}
-			
+
 			case HYPERLINK_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetHyperlink(buf, icp);
 			}
-			
+
 			case INCLUDE_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetInclude(buf, icp);
 			}
-			
+
 			case INDEX_CODE: {
-				docstring arg = cmd.argument();
+				docstring const & arg = cmd.argument();
 				return new InsetIndex(buf, InsetIndexParams(arg));
 			}
-			
+
 			case INDEX_PRINT_CODE:  {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetPrintIndex(buf, icp);
 			}
-			
+
 			case LABEL_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetLabel(buf, icp);
 			}
-			
+
 			case LINE_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetLine(buf, icp);
 			}
-				
+
 			case LISTINGS_CODE: {
 				InsetListingsParams par;
 				InsetListings::string2params(to_utf8(cmd.argument()), par);
 				return new InsetListings(buf, par);
 			}
-			
+
 			case NOMENCL_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetNomencl(buf, icp);
 			}
-			
+
 			case REF_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
@@ -387,13 +403,13 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 				InsetSpace::string2params(to_utf8(cmd.argument()), isp);
 				return new InsetSpace(isp);
 			}
-			
+
 			case TOC_CODE: {
 				InsetCommandParams icp(code);
 				InsetCommand::string2params(to_utf8(cmd.argument()), icp);
 				return new InsetTOC(buf, icp);
 			}
-			
+
 			case VSPACE_CODE: {
 				VSpace vspace;
 				InsetVSpace::string2params(to_utf8(cmd.argument()), vspace);
@@ -402,12 +418,12 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 
 			case PREVIEW_CODE:
 				return new InsetPreview(buf);
-			
+
 			default:
 				lyxerr << "Inset '" << name << "' not permitted with LFUN_INSET_INSERT."
 						<< endl;
-				return 0;
-			
+				return nullptr;
+
 			}
 		} //end LFUN_INSET_INSERT
 
@@ -478,7 +494,6 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 			}
 			return new InsetSpace(isp);
 		}
-		break;
 
 		default:
 			break;
@@ -491,11 +506,11 @@ Inset * createInsetHelper(Buffer * buf, FuncRequest const & cmd)
 			lyx_exit(1);
 		} else if (message.type_ == WarningException) {
 			Alert::warning(message.title_, message.details_);
-			return 0;
+			return nullptr;
 		}
 	}
 
-	return 0;
+	return nullptr;
 }
 
 
@@ -514,13 +529,13 @@ Inset * readInset(Lexer & lex, Buffer * buf)
 	if (lex.getString() != "\\begin_inset")
 		LYXERR0("Buffer::readInset: Consistency check failed.");
 
-	auto_ptr<Inset> inset;
+	unique_ptr<Inset> inset;
 
 	string tmptok;
 	lex >> tmptok;
 
 	// test the different insets
-	
+
 	// FIXME It would be better if we did not have this branch and could
 	// just do one massive switch for all insets. But at present, it's
 	// easier to do it this way, and we can't do the massive switch until
@@ -532,9 +547,9 @@ Inset * readInset(Lexer & lex, Buffer * buf)
 		lex.next();
 		string const insetType = lex.getString();
 		lex.pushToken(insetType);
-		
+
 		InsetCode const code = insetCode(insetType);
-		
+
 		//FIXME If we do the one massive switch, we cannot do this here, since
 		//we do not know in advance that we're dealing with a command inset.
 		//Worst case, we could put it in each case below. Better, we could
@@ -549,9 +564,12 @@ Inset * readInset(Lexer & lex, Buffer * buf)
 			case BIBTEX_CODE:
 				inset.reset(new InsetBibtex(buf, inscmd));
 				break;
-			case CITE_CODE: 
+			case CITE_CODE:
 				inset.reset(new InsetCitation(buf, inscmd));
 				break;
+			case COUNTER_CODE:
+				inset.reset(new InsetCounter(buf, inscmd));
+				break;
 			case HYPERLINK_CODE:
 				inset.reset(new InsetHyperlink(buf, inscmd));
 				break;
@@ -575,7 +593,7 @@ Inset * readInset(Lexer & lex, Buffer * buf)
 				break;
 			case REF_CODE:
 				if (inscmd["name"].empty() && inscmd["reference"].empty())
-					return 0;
+					return nullptr;
 				inset.reset(new InsetRef(buf, inscmd));
 				break;
 			case TOC_CODE:
@@ -587,10 +605,10 @@ Inset * readInset(Lexer & lex, Buffer * buf)
 							<< "'" << endl;
 				while (lex.isOK() && lex.getString() != "\\end_inset")
 					lex.next();
-				return 0;
+				return nullptr;
 		}
 		inset->setBuffer(*buf);
-	} else { 
+	} else {
 		// FIXME This branch should be made to use inset codes
 		// as the preceding branch does. Unfortunately, that
 		// will take some doing. It requires converting the
@@ -605,7 +623,7 @@ Inset * readInset(Lexer & lex, Buffer * buf)
 		} else if (tmptok == "External") {
 			inset.reset(new InsetExternal(buf));
 		} else if (tmptok == "FormulaMacro") {
-			inset.reset(new MathMacroTemplate(buf));
+			inset.reset(new InsetMathMacroTemplate(buf));
 		} else if (tmptok == "Formula") {
 			inset.reset(new InsetMathHull(buf));
 		} else if (tmptok == "Graphics") {
@@ -652,14 +670,17 @@ Inset * readInset(Lexer & lex, Buffer * buf)
 			inset.reset(new InsetFloat(buf, string()));
 		} else if (tmptok == "Wrap") {
 			lex.next();
-			string tmptok = lex.getString();
-			inset.reset(new InsetWrap(buf, tmptok));
+			string tmptok2 = lex.getString();
+			inset.reset(new InsetWrap(buf, tmptok2));
 		} else if (tmptok == "Caption") {
 			lex.eatLine();
 			string s = lex.getString();
 			inset.reset(new InsetCaption(buf, s));
 		} else if (tmptok == "Index") {
 			inset.reset(new InsetIndex(buf, InsetIndexParams()));
+		} else if (tmptok == "IndexMacro") {
+			string s = lex.getString();
+			inset.reset(new InsetIndexMacro(buf, s));
 		} else if (tmptok == "FloatList") {
 			inset.reset(new InsetFloatList(buf));
 		} else if (tmptok == "Info") {
@@ -675,11 +696,11 @@ Inset * readInset(Lexer & lex, Buffer * buf)
 			       << "'" << endl;
 			while (lex.isOK() && lex.getString() != "\\end_inset")
 				lex.next();
-			return 0;
+			return nullptr;
 		}
 
 		// Set the buffer reference for proper parsing of some insets
-		// (InsetCollapsable for example)
+		// (InsetCollapsible for example)
 		inset->setBuffer(*buf);
 		inset->read(lex);
 		// Set again the buffer for insets that are created inside this inset
