@@ -19,8 +19,10 @@ namespace lyx {
 namespace frontend {
 namespace Alert {
 
+typedef unsigned short buttonid;
+
 /**
- * Prompt for a question. Returns 0-2 for the chosen button.
+ * Prompt for a question. Returns 0-3 for the chosen button.
  * Set default_button and cancel_button to reasonable values. b1-b3
  * should have accelerators marked with an '&'. title should be
  * a short summary. Strings should be gettextised.
@@ -30,11 +32,11 @@ namespace Alert {
  * "Yes" or "No", I will personally come around to your house and
  * slap you with fish, and not in an enjoyable way either.
  */
-int prompt(docstring const & title, docstring const & question,
-	   int default_button, int cancel_button,
-	   docstring const & b1, docstring const & b2,
-	   docstring const & b3 = empty_docstring(),
-	   docstring const & b4 = empty_docstring());
+buttonid prompt(docstring const & title, docstring const & question,
+	   buttonid default_button, buttonid cancel_button,
+	   docstring const & b0, docstring const & b1,
+	   docstring const & b2 = empty_docstring(),
+	   docstring const & b3 = empty_docstring());
 
 /**
  * Display a warning to the user. Title should be a short (general) summary.
@@ -45,7 +47,7 @@ int prompt(docstring const & title, docstring const & question,
  * The console output takes care of converting any Qt html to plain text.
  */
 void warning(docstring const & title, docstring const & message,
-	     bool const & askshowagain = false);
+	     bool askshowagain = false);
 
 /**
  * Display a warning to the user. Title should be a short (general) summary.
