@@ -10,10 +10,11 @@
 
 #include <config.h>
 
-#include <cstdlib>
-#include <iostream>
 #include <boost/assert.hpp>
 
+#include <cstdlib>
+#include <exception>
+
 using namespace std;
 
 namespace boost {
@@ -21,7 +22,6 @@ namespace boost {
 #ifndef BOOST_NO_EXCEPTIONS
 void throw_exception(exception const & /*e*/)
 {
-	cerr<<"ASSERTION VIOLATED IN biblio test";
 	BOOST_ASSERT(false);
 }
 #endif
@@ -33,4 +33,11 @@ void assertion_failed(char const * /*expr*/, char const * /*function*/,
 	::abort();
 }
 
+
+void assertion_failed_msg(char const * /*expr*/, char const * /*msg*/,
+						  char const * /*function*/, char const * /*file*/, long /*line*/)
+{
+	::abort();
+}
+
 } // namespace boost
