@@ -25,22 +25,21 @@ namespace lyx {
 class InsetMarginal : public InsetFootlike {
 public:
 	///
-	InsetMarginal(Buffer *);
+	explicit InsetMarginal(Buffer *);
 	///
-	InsetCode lyxCode() const { return MARGIN_CODE; }
+	InsetCode lyxCode() const override { return MARGIN_CODE; }
 	///
-	docstring layoutName() const { return from_ascii("Marginal"); }
+	docstring layoutName() const override { return from_ascii("Marginal"); }
 	///
 	int plaintext(odocstringstream & ods, OutputParams const & op,
-	              size_t max_length = INT_MAX) const;
+	              size_t max_length = INT_MAX) const override;
 	///
-	int docbook(odocstream &, OutputParams const & runparams) const;
-	///
-	void addToToc(DocIterator const & di, bool output_active,
-				  UpdateType utype) const;
+	void docbook(XMLStream &, OutputParams const & runparams) const override;
+	/// Is the content of this inset part of the immediate (visible) text sequence?
+	bool isPartOfTextSequence() const override { return false; }
 private:
 	///
-	Inset * clone() const { return new InsetMarginal(*this); }
+	Inset * clone() const override { return new InsetMarginal(*this); }
 };
 
 
