@@ -21,6 +21,11 @@
 #include <ios>
 #endif
 
+#ifdef WIN32
+#include <Windows.h>
+#endif
+
+
 using namespace std;
 
 
@@ -30,6 +35,14 @@ int main(int argc, char * argv[])
 	ios_base::sync_with_stdio(false);
 #endif
 
+#ifdef WIN32
+	if (AttachConsole(ATTACH_PARENT_PROCESS)) {
+		freopen("CONOUT$", "w", stdout);
+		freopen("CONOUT$", "w", stderr);
+	}
+	SetConsoleOutputCP(CP_UTF8);
+#endif
+
 	// To avoid ordering of global object problems with some
 	// stdlibs we do the initialization here, but still as
 	// early as possible.
