@@ -92,8 +92,8 @@ static bool slanted(char_type c)
 }
 
 
-InsetMathChar::InsetMathChar(char_type c)
-	: char_(c), kerning_(0), subst_(makeSubstitute(c))
+InsetMathChar::InsetMathChar(Buffer * buf, char_type c)
+	: InsetMath(buf), char_(c), kerning_(0), subst_(makeSubstitute(c))
 {}
 
 
@@ -248,7 +248,7 @@ void InsetMathChar::mathmlize(MathMLStream & ms) const
 
 	if (ms.inText()) {
 		if (entity.empty())
-			ms.os().put(char_);
+			ms << char_;
 		else
 			ms << from_ascii(entity);
 		return;
