@@ -13,16 +13,12 @@
 #include "InsetMathScript.h"
 
 #include "InsetMathBrace.h"
-#include "InsetMathSymbol.h"
 #include "MathData.h"
 #include "MathStream.h"
 #include "MathSupport.h"
 
 #include "BufferView.h"
 #include "Cursor.h"
-#include "DispatchResult.h"
-#include "FuncRequest.h"
-#include "FuncStatus.h"
 #include "LaTeXFeatures.h"
 #include "MetricsInfo.h"
 
@@ -37,17 +33,17 @@ namespace lyx {
 
 
 InsetMathScript::InsetMathScript(Buffer * buf)
-	: InsetMathNest(buf, 1), cell_1_is_up_(false), limits_(0)
+	: InsetMathNest(buf, 1), cell_1_is_up_(false)
 {}
 
 
 InsetMathScript::InsetMathScript(Buffer * buf, bool up)
-	: InsetMathNest(buf, 2), cell_1_is_up_(up), limits_(0)
+	: InsetMathNest(buf, 2), cell_1_is_up_(up)
 {}
 
 
 InsetMathScript::InsetMathScript(Buffer * buf, MathAtom const & at, bool up)
-	: InsetMathNest(buf, 2), cell_1_is_up_(up), limits_(0)
+	: InsetMathNest(buf, 2), cell_1_is_up_(up)
 {
 	LATTEST(nargs() >= 1);
 	cell(0).push_back(at);
@@ -72,22 +68,6 @@ InsetMathScript * InsetMathScript::asScriptInset()
 }
 
 
-bool InsetMathScript::idxFirst(Cursor & cur) const
-{
-	cur.idx() = 0;
-	cur.pos() = 0;
-	return true;
-}
-
-
-bool InsetMathScript::idxLast(Cursor & cur) const
-{
-	cur.idx() = 0;
-	cur.pos() = nuc().size();
-	return true;
-}
-
-
 MathData const & InsetMathScript::down() const
 {
 	if (nargs() == 3)
@@ -124,14 +104,14 @@ void InsetMathScript::ensure(bool up)
 {
 	if (nargs() == 1) {
 		// just nucleus so far
-		cells_.push_back(MathData());
+		cells_.push_back(MathData(buffer_));
 		cell_1_is_up_ = up;
 	} else if (nargs() == 2 && !has(up)) {
 		if (up) {
 			cells_.push_back(cell(1));
 			cell(1).clear();
 		} else {
-			cells_.push_back(MathData());
+			cells_.push_back(MathData(buffer_));
 		}
 	}
 }
@@ -195,7 +175,7 @@ int InsetMathScript::dy0(BufferView const & bv) const
 	if (!hasDown())
 		return nd;
 	int des = down().dimension(bv).ascent();
-	if (hasLimits())
+	if (has_limits_)
 		des += nd + 2;
 	else {
 		int na = nasc(bv);
@@ -211,7 +191,7 @@ int InsetMathScript::dy1(BufferView const & bv) const
 	if (!hasUp())
 		return na;
 	int asc = up().dimension(bv).descent();
-	if (hasLimits())
+	if (has_limits_)
 		asc += na + 2;
 	else {
 		int nd = ndes(bv);
@@ -226,7 +206,7 @@ int InsetMathScript::dx0(BufferView const & bv) const
 {
 	LASSERT(hasDown(), return 0);
 	Dimension const dim = dimension(bv);
-	return hasLimits() ? (dim.wid - down().dimension(bv).width()) / 2
+	return has_limits_ ? (dim.wid - down().dimension(bv).width()) / 2
 		: nwid(bv) + min(nker(&bv), 0);
 }
 
@@ -235,7 +215,7 @@ int InsetMathScript::dx1(BufferView const & bv) const
 {
 	LASSERT(hasUp(), return 0);
 	Dimension const dim = dimension(bv);
-	return hasLimits() ? (dim.wid - up().dimension(bv).width()) / 2
+	return has_limits_ ? (dim.wid - up().dimension(bv).width()) / 2
 		: nwid(bv) + max(nker(&bv), 0);
 }
 
@@ -243,7 +223,7 @@ int InsetMathScript::dx1(BufferView const & bv) const
 int InsetMathScript::dxx(BufferView const & bv) const
 {
 	Dimension const dim = dimension(bv);
-	return hasLimits() ? (dim.wid - nwid(bv)) / 2  :  0;
+	return has_limits_ ? (dim.wid - nwid(bv)) / 2  :  0;
 }
 
 
@@ -273,6 +253,23 @@ int InsetMathScript::nker(BufferView const * bv) const
 }
 
 
+Limits InsetMathScript::limits() const
+{
+	if (nuc().empty())
+		return AUTO_LIMITS;
+	else
+		// only the limits status of the last element counts
+		return nuc().back()->limits();
+}
+
+
+void InsetMathScript::limits(Limits lim)
+{
+	if (!nuc().empty())
+		nuc().back()->limits(lim);
+}
+
+
 MathClass InsetMathScript::mathClass() const
 {
 	// FIXME: this is a hack, since the class will not be correct if
@@ -291,16 +288,23 @@ MathClass InsetMathScript::mathClass() const
 
 void InsetMathScript::metrics(MetricsInfo & mi, Dimension & dim) const
 {
-	Changer dummy2 = mi.base.changeEnsureMath();
+	// we store this, because it is much easier
+	has_limits_ = hasLimits(mi.base.font.style());
+
+	// Compute metrics of the available cells
 	Dimension dim0;
 	Dimension dim1;
 	Dimension dim2;
-	cell(0).metrics(mi, dim0);
-	Changer dummy = mi.base.changeScript();
-	if (nargs() > 1)
-		cell(1).metrics(mi, dim1, !hasLimits());
-	if (nargs() > 2)
-		cell(2).metrics(mi, dim2, !hasLimits());
+	{
+		// Keeps the changers local
+		Changer dummy2 = mi.base.changeEnsureMath();
+		cell(0).metrics(mi, dim0);
+		Changer dummy = mi.base.changeScript();
+		if (nargs() > 1)
+			cell(1).metrics(mi, dim1, !has_limits_);
+		if (nargs() > 2)
+			cell(2).metrics(mi, dim2, !has_limits_);
+	}
 
 	dim.wid = 0;
 	BufferView & bv = *mi.base.bv;
@@ -312,7 +316,7 @@ void InsetMathScript::metrics(MetricsInfo & mi, Dimension & dim) const
 	if (hasDown())
 		dimdown = down().dimension(bv);
 
-	if (hasLimits()) {
+	if (has_limits_) {
 		dim.wid = nwid(bv);
 		if (hasUp())
 			dim.wid = max(dim.wid, dimup.width());
@@ -381,47 +385,16 @@ void InsetMathScript::drawT(TextPainter & pain, int x, int y) const
 }
 
 
-// FIXME: See InsetMathSymbol::takesLimits, which seems to attempt the
-// same in a hardcoded way. takeLimits use is currently commented out in
-// InsetMathScript::metrics. It seems that the mathop test is general
-// enough, but only time will tell.
-bool InsetMathScript::allowsLimits() const
+bool InsetMathScript::hasLimits(MathStyle const & style) const
 {
 	if (nuc().empty())
 		return false;
-	// Only makes sense for insets of mathop class
-	if (nuc().back()->mathClass() != MC_OP)
-		return false;
-	return true;
-}
 
-
-bool InsetMathScript::hasLimits() const
-{
-	// obvious cases
-	if (limits_ == 1)
-		return true;
-	if (limits_ == -1)
-		return false;
-
-	// we can only display limits if the nucleus wants some
-	if (!allowsLimits())
-		return false;
-	// FIXME: this is some hardcoding done in InsetMathSymbol::metrics.
-	if (!nuc().back()->isScriptable())
-		return false;
-
-	if (nuc().back()->asSymbolInset()) {
-		// \intop is an alias for \int\limits, \ointop == \oint\limits
-		if (nuc().back()->asSymbolInset()->name().find(from_ascii("intop")) != string::npos)
-			return true;
-		// per default \int has limits beside the \int even in displayed formulas
-		if (nuc().back()->asSymbolInset()->name().find(from_ascii("int")) != string::npos)
-			return false;
-	}
-
-	// assume "real" limits for everything else
-	return true;
+	Limits const lim = nuc().back()->limits() == AUTO_LIMITS
+		? nuc().back()->defaultLimits(style == DISPLAY_STYLE)
+		: nuc().back()->limits();
+	LASSERT(lim != AUTO_LIMITS, return false);
+	return lim == LIMITS;
 }
 
 
@@ -431,7 +404,7 @@ void InsetMathScript::removeScript(bool up)
 		if (up == cell_1_is_up_)
 			cells_.pop_back();
 	} else if (nargs() == 3) {
-		if (up == true) {
+		if (up) {
 			swap(cells_[1], cells_[2]);
 			cell_1_is_up_ = false;
 		} else {
@@ -464,7 +437,7 @@ bool InsetMathScript::hasDown() const
 }
 
 
-Inset::idx_type InsetMathScript::idxOfScript(bool up) const
+idx_type InsetMathScript::idxOfScript(bool up) const
 {
 	if (nargs() == 1)
 		return 0;
@@ -497,7 +470,8 @@ bool InsetMathScript::idxUpDown(Cursor & cur, bool up) const
 			return false;
 		// go up/down only if in the last position
 		// or in the first position of something with displayed limits
-		if (cur.pos() == cur.lastpos() || (cur.pos() == 0 && hasLimits())) {
+		if (cur.pos() == cur.lastpos()
+		                 || (cur.pos() == 0 && has_limits_)) {
 			cur.idx() = idxOfScript(up);
 			cur.pos() = 0;
 			return true;
@@ -531,24 +505,19 @@ bool InsetMathScript::idxUpDown(Cursor & cur, bool up) const
 }
 
 
-void InsetMathScript::write(WriteStream & os) const
+void InsetMathScript::write(TeXMathStream & os) const
 {
 	MathEnsurer ensurer(os);
 
 	if (!nuc().empty()) {
 		os << nuc();
-		//if (nuc().back()->takesLimits()) {
-			if (limits_ == -1)
-				os << "\\nolimits ";
-			if (limits_ == 1)
-				os << "\\limits ";
-		//}
-	} else {
-		if (os.firstitem())
-			LYXERR(Debug::MATHED, "suppressing {} when writing");
-		else
+		// Avoid double superscript errors (bug 1633)
+		if (os.latex() && hasUp() && nuc().back()->getChar() == '\'')
 			os << "{}";
-	}
+	} else if (os.firstitem())
+		LYXERR(Debug::MATHED, "suppressing {} when writing");
+	else
+		os << "{}";
 
 	if (hasDown() /*&& !down().empty()*/)
 		os << "_{" << down() << '}';
@@ -560,8 +529,13 @@ void InsetMathScript::write(WriteStream & os) const
 		    (up().size() == 1 && up().back()->asBraceInset() &&
 		     up().back()->asBraceInset()->cell(0).empty())))
 			os << "^ {}";
-		else
+		else {
 			os << "^{" << up() << '}';
+			// Avoid double superscript errors by writing an
+			// empty group {} when a prime immediately follows
+			if (os.latex())
+				os.useBraces(true);
+		}
 	}
 
 	if (lock_ && !os.latex())
@@ -628,32 +602,39 @@ void InsetMathScript::mathematica(MathematicaStream & os) const
 }
 
 
-void InsetMathScript::mathmlize(MathStream & os) const
+void InsetMathScript::mathmlize(MathMLStream & ms) const
 {
 	bool d = hasDown() && !down().empty();
 	bool u = hasUp() && !up().empty();
-	bool l = hasLimits();
+	bool has_limits = hasLimits(ms.getFontMathStyle());
 
+	if (!d && !u)
+		return;
+
+	const char * tag = nullptr;
 	if (u && d)
-		os << MTag(l ? "munderover" : "msubsup");
+		tag = has_limits ? "munderover" : "msubsup";
 	else if (u)
-		os << MTag(l ? "mover" : "msup");
+		tag = has_limits ? "mover" : "msup";
 	else if (d)
-		os << MTag(l ? "munder" : "msub");
+		tag = has_limits ? "munder" : "msub";
+
+	ms << MTag(tag);
 
 	if (!nuc().empty())
-		os << MTag("mrow") << nuc() << ETag("mrow");
+		ms << nuc();
 	else
-		os << "<mrow />";
+		// TODO: is this empty <mrow> required?
+		ms << CTag("mrow");
 
-	if (u && d)
-		os << MTag("mrow") << down() << ETag("mrow")
-		   << MTag("mrow") << up() << ETag("mrow")
-		   << ETag(l ? "munderover" : "msubsup");
-	else if (u)
-		os << MTag("mrow") << up() << ETag("mrow") << ETag(l ? "mover" : "msup");
-	else if (d)
-		os << MTag("mrow") << down() << ETag("mrow") << ETag(l ? "munder" : "msub");
+	// No need to wrap these in an <mrow>, as it's done by MathExtern.
+	// More details in https://www.lyx.org/trac/ticket/12221#comment:10.
+	if (d)
+		ms << down();
+	if (u)
+		ms << up();
+
+	ms << ETag(tag);
 }
 
 
@@ -696,8 +677,7 @@ void InsetMathScript::infoize(odocstream & os) const
 
 void InsetMathScript::infoize2(odocstream & os) const
 {
-	if (limits_)
-		os << from_ascii(limits_ == 1 ? ", Displayed limits" : ", Inlined limits");
+	os << from_ascii(has_limits_ == 1 ? ", Displayed limits" : ", Inlined limits");
 }
 
 
@@ -761,57 +741,6 @@ bool InsetMathScript::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 }
 
 
-void InsetMathScript::doDispatch(Cursor & cur, FuncRequest & cmd)
-{
-	//LYXERR("InsetMathScript: request: " << cmd);
-
-	if (cmd.action() == LFUN_MATH_LIMITS) {
-		// only when nucleus allows this
-		if (!allowsLimits())
-			return;
-		cur.recordUndoInset();
-		if (!cmd.argument().empty()) {
-			if (cmd.argument() == "limits")
-				limits_ = 1;
-			else if (cmd.argument() == "nolimits")
-				limits_ = -1;
-			else
-				limits_ = 0;
-		} else if (limits_ == 0)
-			limits_ = hasLimits() ? -1 : 1;
-		else
-			limits_ = 0;
-		return;
-	}
-
-	InsetMathNest::doDispatch(cur, cmd);
-}
-
-
-bool InsetMathScript::getStatus(Cursor & cur, FuncRequest const & cmd,
-				FuncStatus & flag) const
-{
-	if (cmd.action() == LFUN_MATH_LIMITS) {
-		// only when nucleus allows this
-		if (allowsLimits()) {
-			if (!cmd.argument().empty()) {
-				if (cmd.argument() == "limits")
-					flag.setOnOff(limits_ == 1);
-				else if (cmd.argument() == "nolimits")
-					flag.setOnOff(limits_ == -1);
-				else
-					flag.setOnOff(limits_ == 0);
-			}
-			flag.setEnabled(true);
-		} else
-			flag.setEnabled(false);
-		return true;
-	}
-
-	return InsetMathNest::getStatus(cur, cmd, flag);
-}
-
-
 // the idea for dual scripts came from the eLyXer code
 void InsetMathScript::validate(LaTeXFeatures & features) const
 {
