@@ -12,9 +12,10 @@
 #ifndef MATH_XYMATRIX_H
 #define MATH_XYMATRIX_H
 
-#include "Length.h"
 #include "InsetMathGrid.h"
 
+#include "support/Length.h"
+
 
 namespace lyx {
 
@@ -25,36 +26,36 @@ public:
 	InsetMathXYMatrix(Buffer * buf, Length const & = Length(), char c = '\0',
 		bool equal_spacing = false);
 	///
-	void metrics(MetricsInfo &, Dimension &) const;
+	void metrics(MetricsInfo &, Dimension &) const override;
 	///
-	void draw(PainterInfo & pi, int x, int y) const;
+	void draw(PainterInfo & pi, int x, int y) const override;
 	///
 	InsetMathXYMatrix const * asXYMatrixInset() const { return this; }
 	///
-	virtual int colsep() const;
+	int colsep() const override;
 	///
-	virtual int rowsep() const;
+	int rowsep() const override;
 
 	///
-	void write(WriteStream & os) const;
+	void write(TeXMathStream & os) const override;
 	///
-	void infoize(odocstream & os) const;
+	void infoize(odocstream & os) const override;
 	///
-	void normalize(NormalStream &) const;
+	void normalize(NormalStream &) const override;
 	///
-	void maple(MapleStream &) const;
+	void maple(MapleStream &) const override;
 	///
-	void validate(LaTeXFeatures & features) const;
+	void validate(LaTeXFeatures & features) const override;
 	///
-	InsetCode lyxCode() const { return MATH_XYMATRIX_CODE; }
+	InsetCode lyxCode() const override { return MATH_XYMATRIX_CODE; }
 	///
-	void mathmlize(MathStream &) const;
+	void mathmlize(MathMLStream &) const override;
 	///
-	void htmlize(HtmlStream &) const;
+	void htmlize(HtmlStream &) const override;
 
 private:
 	///
-	virtual Inset * clone() const;
+	Inset * clone() const override;
 	/// extra spacing, may be empty
 	Length spacing_;
 	///
