@@ -9,8 +9,8 @@
  * Full author contact details are available in file CREDITS.
  */
 
-#ifndef MATH_MATHMLSTREAM_H
-#define MATH_MATHMLSTREAM_H
+#ifndef MATH_MATHSTREAM_H
+#define MATH_MATHSTREAM_H
 
 #include "InsetMath.h"
 #include "FontInfo.h"
@@ -380,22 +380,16 @@ public:
 	explicit MathMLStream(odocstream & os, std::string const & xmlns = "");
 	///
 	void cr();
-	///
-	odocstream & os() { return os_; }
-	///
-	int line() const { return line_; }
-	///
+	/// Indentation when nesting tags
 	int & tab() { return tab_; }
 	///
-	friend MathMLStream & operator<<(MathMLStream &, char const *);
-	///
 	void defer(docstring const &);
 	///
 	void defer(std::string const &);
 	///
 	docstring deferred() const;
 	///
-	bool inText() const { return in_text_; }
+	bool inText() const { return text_level_ != nlevel; }
 	///
 	std::string xmlns() const { return xmlns_; }
 	/// Returns the tag name prefixed by the name space if needed.
@@ -407,16 +401,21 @@ public:
 	/// Returns the current math style in the stream.
 	void setFontMathStyle(const MathStyle style) { font_math_style_ = style; }
 private:
-	///
-	void setTextMode(bool t) { in_text_ = t; }
+	/// Check whether it makes sense to start a <mtext>
+	void beforeText();
+	///Check whether there is a <mtext> to close here
+	void beforeTag();
 	///
 	odocstream & os_;
 	///
-	int tab_;
+	int tab_ = 0;
 	///
-	int line_;
+	int nesting_level_ = 0;
+	static const int nlevel = -1000;
 	///
-	bool in_text_;
+	int text_level_ = nlevel;
+	///
+	bool in_mtext_ = false;
 	///
 	odocstringstream deferred_;
 	///
@@ -425,6 +424,14 @@ private:
 	MathStyle font_math_style_;
 	///
 	friend class SetMode;
+	friend MathMLStream & operator<<(MathMLStream &, MathAtom const &);
+	friend MathMLStream & operator<<(MathMLStream &, MathData const &);
+	friend MathMLStream & operator<<(MathMLStream &, docstring const &);
+	friend MathMLStream & operator<<(MathMLStream &, MTag const &);
+	friend MathMLStream & operator<<(MathMLStream &, MTagInline const &);
+	friend MathMLStream & operator<<(MathMLStream &, ETag const &);
+	friend MathMLStream & operator<<(MathMLStream &, ETagInline const &);
+	friend MathMLStream & operator<<(MathMLStream &, CTag const &);
 };
 
 ///
@@ -462,7 +469,7 @@ private:
 	///
 	MathMLStream & ms_;
 	///
-	bool was_text_;
+	int old_text_level_;
 };
 
 
